## SIMULATION: HETEROSKEDASTICITY
set.seed(102)
x=runif(20000)
#g=function(x) (x-x^2)
g=function(x) sin(x*3*pi)^2
y=sapply(x,function(x) rnorm(1)*g(x))
win.graph()
par(mfrow=c(1,3))
plot(g,0,1,type='l',ylab='sd(Y)',xlab='X',main='sd(Y|X)=g(X)')
plot(x,y,pch='.',xlab='X',ylab='Y',main='Y=Z*g(X)')
abline(lm(y~x),col='red')

B=500
NN=c(50,100,200,300,400,500)
alpha=0.1
PVALUES=lapply(NN,function(n) {
	sams=lapply(1:B,function(i) {
		x=runif(n)
		y=sapply(x,function(x) rnorm(1)*g(x))
		print(c(n,i))
		return(cbind(x,y))
		})
	Pearson=sapply(1:B,function(i) cor.test(sams[[i]][,1],sams[[i]][,2])$p.value)
	Spearman=sapply(1:B,function(i) cor.test(sams[[i]][,1],sams[[i]][,2],method='spearman')$p.value)
	Kendall=sapply(1:B,function(i) cor.test(sams[[i]][,1],sams[[i]][,2],method='kendall')$p.value)
	Dcov=sapply(1:B,function(i) dcov.test(sams[[i]][,1],sams[[i]][,2],R=500)$p.value)
return(cbind(Pearson,Spearman,Kendall,Dcov))
}
)

POWER=matrix(NA,nr=4,nc=length(NN))
for(j in 1:4){
POWER[j,]=sapply(1:length(NN),function(i) mean((PVALUES[[i]][,j]<alpha)))
}


plot(NN,POWER[1,],type='b',ylim=c(0,0.7),xlab='sample size',ylab='power')
lines(NN,POWER[2,],type='b',col='red')
lines(NN,POWER[3,],type='b',col='blue')
lines(NN,POWER[4,],type='b',col='green',lwd=2)
legend(50,0.7,c('Pearson','Spearman','Kendall','dCov'),col=c('black','red','blue','green'),lwd=c(1,1,1,2))
