# code for asymptotic dcov test

T2=function(x,y){
Dx=dist(x)
Dy=dist(y)
return(sum(Dx)*sum(Dy)/(length(x)^4))
}

asy.test.stat=function(x,y){
length(x)*(dcov(x,y)^2)/T2(x,y)
}

asy.dcov.test=function(x,y,alpha=0.05){
stat=asy.test.stat(x,y)
thr=qchisq(1-alpha,df=1)
return(list(res=round((stat>=thr)),p.value=1-pchisq(stat,df=1)))
}



### SIMULATION: LACK OF MONOTONICITY

## Plots describing the experiment
library(energy)
set.seed(1238)
x=runif(500,0,1)
lambdas=seq(0,11,len=100)[-1]
pears=sapply(1:length(lambdas), function(i) cor(x,sin(lambdas[i]*x*pi/2)))
spear=sapply(1:length(lambdas), function(i) cor(x,sin(lambdas[i]*x*pi/2),method='spearman'))
kendall=sapply(1:length(lambdas), function(i) cor(x,sin(lambdas[i]*x*pi/2),method='kendall'))
dCor=sapply(1:length(lambdas), function(i) dcor(x,sin(lambdas[i]*x*pi/2)))

win.graph()
x=seq(0,1,len=100)
par(mfrow=c(1,2))
plot(x,sin(x*pi/2),main=expression(f[lambda](x)), type='l',ylim=c(-1,1),ylab=expression(f[lambda](x)))
lines(x,sin(x*2*pi/2),main='lambda=2',col='red')
lines(x,sin(x*3*pi/2),main='lambda=3', col='blue')
lines(x,sin(x*4*pi/2),main='lambda=4', col='green')
lines(x,sin(x*6*pi/2),main='lambda=4', col='violet')
abline(h=0,col='grey')
legend(0,-0.2,c(1,2,3,4,6),title=expression(lambda),col=c('black','red','blue','green','violet'),lty=c(1,1,1,1,1))

plot(lambdas,abs(pears),ylim=c(0,1),type='l',ylab='|correlation|',xlab=expression(lambda),main='Absolute value of correlations')
lines(lambdas,abs(spear),col='red')
lines(lambdas,abs(kendall),col='blue')
lines(lambdas,abs(dCor),col='green',lwd=2)
legend(7.5,1,c('Pearson','Spearman','Kendall','dCor'),col=c('black','red','blue','green'),lwd=c(1,1,1,2))


## Simulations
library(energy)
B=300
NN=c(1:10)*10
sigeps=0.5


### H0 true ###
set.seed(1005)
xx=lapply(NN,function(n) lapply(1:B,function(i) runif(n,0,1)))
eps=lapply(NN,function(n) lapply(1:B,function(i) rnorm(n)*sigeps))
yy1=lapply(1:length(xx),function(n) lapply(1:B,function(i) sin(runif(n,0,1)*1*pi/2)+eps[[n]][[i]]))
yy2=lapply(1:length(xx),function(n) lapply(1:B,function(i) sin(runif(n,0,1)*2*pi/2)+eps[[n]][[i]]))

# pvalues
pears1.0=lapply(1:length(xx),function(n) sapply(1:B,function(i) cor.test(xx[[n]][[i]],yy1[[n]][[i]])$p.value))
pears2.0=lapply(1:length(xx),function(n) sapply(1:B,function(i) cor.test(xx[[n]][[i]],yy2[[n]][[i]])$p.value))
spear1.0=lapply(1:length(xx),function(n) sapply(1:B,function(i) cor.test(xx[[n]][[i]],yy1[[n]][[i]],method='spearman')$p.value))
spear2.0=lapply(1:length(xx),function(n) sapply(1:B,function(i) cor.test(xx[[n]][[i]],yy2[[n]][[i]],method='spearman')$p.value))
kend1.0=lapply(1:length(xx),function(n) sapply(1:B,function(i) cor.test(xx[[n]][[i]],yy1[[n]][[i]],method='kendall')$p.value))
kend2.0=lapply(1:length(xx),function(n) sapply(1:B,function(i) cor.test(xx[[n]][[i]],yy2[[n]][[i]],method='kendall')$p.value))
dCov1.0=lapply(1:length(xx),function(n) sapply(1:B,function(i) dcov.test(xx[[n]][[i]],yy1[[n]][[i]])$p.value))
dCov2.0=lapply(1:length(xx),function(n) sapply(1:B,function(i) dcov.test(xx[[n]][[i]],yy2[[n]][[i]])$p.value))
asy.dCov1.0=lapply(1:length(xx),function(n) sapply(1:B,function(i) asy.dcov.test(xx[[n]][[i]],yy1[[n]][[i]])$p.value))
asy.dCov2.0=lapply(1:length(xx),function(n) sapply(1:B,function(i) asy.dcov.test(xx[[n]][[i]],yy2[[n]][[i]])$p.value))

# prob TYPE I error
alpha.lev=0.05
Pears1.0=sapply(1:length(NN), function(n) mean((pears1.0[[n]]<=alpha.lev)))
Pears2.0=sapply(1:length(NN), function(n) mean((pears2.0[[n]]<=alpha.lev)))
Spear1.0=sapply(1:length(NN), function(n) mean((spear1.0[[n]]<=alpha.lev)))
Spear2.0=sapply(1:length(NN), function(n) mean((spear2.0[[n]]<=alpha.lev)))
Kend1.0=sapply(1:length(NN), function(n) mean((kend1.0[[n]]<=alpha.lev)))
Kend2.0=sapply(1:length(NN), function(n) mean((kend2.0[[n]]<=alpha.lev)))
DCov1.0=sapply(1:length(NN), function(n) mean((dCov1.0[[n]]<=alpha.lev)))
DCov2.0=sapply(1:length(NN), function(n) mean((dCov2.0[[n]]<=alpha.lev)))
asy.DCov1.0=sapply(1:length(NN), function(n) mean((asy.dCov1.0[[n]]<=alpha.lev)))
asy.DCov2.0=sapply(1:length(NN), function(n) mean((asy.dCov2.0[[n]]<=alpha.lev)))


### H0 false ###
set.seed(1005)
xx=lapply(NN,function(n) lapply(1:B,function(i) runif(n,0,1)))
eps=lapply(NN,function(n) lapply(1:B,function(i) rnorm(n)*sigeps))
yy1=lapply(1:length(xx),function(n) lapply(1:B,function(i) sin(xx[[n]][[i]]*1*pi/2)+eps[[n]][[i]]))
yy2=lapply(1:length(xx),function(n) lapply(1:B,function(i) sin(xx[[n]][[i]]*2*pi/2)+eps[[n]][[i]]))

# pvalues
pears1=lapply(1:length(xx),function(n) sapply(1:B,function(i) cor.test(xx[[n]][[i]],yy1[[n]][[i]])$p.value))
pears2=lapply(1:length(xx),function(n) sapply(1:B,function(i) cor.test(xx[[n]][[i]],yy2[[n]][[i]])$p.value))
spear1=lapply(1:length(xx),function(n) sapply(1:B,function(i) cor.test(xx[[n]][[i]],yy1[[n]][[i]],method='spearman')$p.value))
spear2=lapply(1:length(xx),function(n) sapply(1:B,function(i) cor.test(xx[[n]][[i]],yy2[[n]][[i]],method='spearman')$p.value))
kend1=lapply(1:length(xx),function(n) sapply(1:B,function(i) cor.test(xx[[n]][[i]],yy1[[n]][[i]],method='kendall')$p.value))
kend2=lapply(1:length(xx),function(n) sapply(1:B,function(i) cor.test(xx[[n]][[i]],yy2[[n]][[i]],method='kendall')$p.value))
dCov1=lapply(1:length(xx),function(n) sapply(1:B,function(i) dcov.test(xx[[n]][[i]],yy1[[n]][[i]])$p.value))
dCov2=lapply(1:length(xx),function(n) sapply(1:B,function(i) dcov.test(xx[[n]][[i]],yy2[[n]][[i]])$p.value))
asy.dCov1=lapply(1:length(xx),function(n) sapply(1:B,function(i) asy.dcov.test(xx[[n]][[i]],yy1[[n]][[i]])$p.value))
asy.dCov2=lapply(1:length(xx),function(n) sapply(1:B,function(i) asy.dcov.test(xx[[n]][[i]],yy2[[n]][[i]])$p.value))

# power functions
alpha.lev=0.05
Pears1=sapply(1:length(NN), function(n) mean((pears1[[n]]<=alpha.lev)))
Pears2=sapply(1:length(NN), function(n) mean((pears2[[n]]<=alpha.lev)))
Spear1=sapply(1:length(NN), function(n) mean((spear1[[n]]<=alpha.lev)))
Spear2=sapply(1:length(NN), function(n) mean((spear2[[n]]<=alpha.lev)))
Kend1=sapply(1:length(NN), function(n) mean((kend1[[n]]<=alpha.lev)))
Kend2=sapply(1:length(NN), function(n) mean((kend2[[n]]<=alpha.lev)))
DCov1=sapply(1:length(NN), function(n) mean((dCov1[[n]]<=alpha.lev)))
DCov2=sapply(1:length(NN), function(n) mean((dCov2[[n]]<=alpha.lev)))
asy.DCov1=sapply(1:length(NN), function(n) mean((asy.dCov1[[n]]<=alpha.lev)))
asy.DCov2=sapply(1:length(NN), function(n) mean((asy.dCov2[[n]]<=alpha.lev)))



### PLOTS ###
win.graph()
par(mfrow=c(2,2))
plot(NN,Pears1,type='b',main='Power function: monotone relationship 
(lambda=1, sig.eps=0.5)',ylim=c(0,1),xlab='sample size',ylab='Power',pch=20)
points(NN,Spear1,type='b',col='red',pch=20)
points(NN,Kend1,type='b',col='blue',pch=20)
points(NN,DCov1,type='b',col='green',pch=20,lwd=2)
points(NN,asy.DCov1,type='b',col='orange',pch=20,lwd=2)
legend(70,0.7,c('Pearson','Spearman','Kendall','perm.dCov','asy.dCov'),col=c('black','red','blue','green','orange'),lwd=c(1,1,1,2))
plot(NN,Pears2,type='b',main='Power function: bell relationship 
(lambda=2, sig.eps=0.5)',ylim=c(0,1),xlab='sample size',ylab='Power',pch=20)
points(NN,Spear2,type='b',col='red',pch=20)
points(NN,Kend2,type='b',col='blue',pch=20)
points(NN,DCov2,type='b',col='green',pch=20,lwd=2)
points(NN,asy.DCov2,type='b',col='orange',pch=20,lwd=2)

plot(NN,Pears1.0,type='b',main='Prob TYPE I error: monotone relationship 
(lambda=1, sig.eps=0.5)',ylim=c(0,1),xlab='sample size',ylab='Power',pch=20)
points(NN,Spear1.0,type='b',col='red',pch=20)
points(NN,Kend1.0,type='b',col='blue',pch=20)
points(NN,DCov1.0,type='b',col='green',pch=20,lwd=2)
points(NN,asy.DCov1.0,type='b',col='orange',pch=20,lwd=2)
legend(70,1,c('Pearson','Spearman','Kendall','perm.dCov','asy.dCov'),col=c('black','red','blue','green','orange'),lwd=c(1,1,1,2))
plot(NN,Pears2.0,type='b',main='Prob TYPE I error: bell relationship 
(lambda=2, sig.eps=0.5)',ylim=c(0,1),xlab='sample size',ylab='Power',pch=20)
points(NN,Spear2.0,type='b',col='red',pch=20)
points(NN,Kend2.0,type='b',col='blue',pch=20)
points(NN,DCov2.0,type='b',col='green',pch=20,lwd=2)
points(NN,asy.DCov2.0,type='b',col='orange',pch=20,lwd=2)



### Power as function of lambda
B=500
NN=50
sigeps=0.5
lambdas=seq(0,12,len=61)[-1]
set.seed(1005)
xx=lapply(1:length(lambdas),function(i) lapply(1:B,function(j) runif(NN,0,1)))
eps=lapply(1:length(lambdas),function(i)  lapply(1:B,function(j) rnorm(NN)*sigeps))
yy=lapply(1:length(lambdas),function(i) lapply(1:B,function(j) sin(xx[[i]][[j]]*lambdas[i]*pi/2)+eps[[i]][[j]]))

# tests
pears=lapply(1:length(lambdas),function(i) sapply(1:B,function(j) cor.test(xx[[i]][[j]],yy[[i]][[j]])$p.value))
spear=lapply(1:length(lambdas),function(i) sapply(1:B,function(j) cor.test(xx[[i]][[j]],yy[[i]][[j]],method='spearman')$p.value))
kend=lapply(1:length(lambdas),function(i) sapply(1:B,function(j) cor.test(xx[[i]][[j]],yy[[i]][[j]],method='kendall')$p.value))
dCov=lapply(1:length(lambdas),function(i) sapply(1:B,function(j) dcov.test(xx[[i]][[j]],yy[[i]][[j]])$p.value))

# power functions
alpha.lev=0.05
Pears=sapply(1:length(lambdas), function(i) mean((pears[[i]]<=alpha.lev)))
Spear=sapply(1:length(lambdas), function(i) mean((spear[[i]]<=alpha.lev)))
Kend=sapply(1:length(lambdas), function(i) mean((kend[[i]]<=alpha.lev)))
DCov=sapply(1:length(lambdas), function(i) mean((dCov[[i]]<=alpha.lev)))

win.graph()
plot(lambdas,Pears,main='Power functions: sig.eps=0.5, sample size=50, 5% level',type='l',ylim=c(0,1),xlab=expression(lambda),ylab='Power',pch=20)
points(lambdas,Spear,type='l',col='red',pch=20)
points(lambdas,Kend,type='l',col='blue',pch=20)
points(lambdas,DCov,type='l',col='green',pch=20,lwd=2)
legend(10.5,1,c('Pearson','Spearman','Kendall','dCov'),col=c('black','red','blue','green'),lwd=c(1,1,1,2))
